@@ -8,9 +8,9 @@ describe('instance', function () {
   });
 
   it('should have the same methods as default instance', function () {
-    var instance = axios.create();
+    const instance = axios.create();
 
-    for (var prop in axios) {
+    for (const prop in axios) {
       if ([
         'Axios',
         'AxiosError',
@@ -25,7 +25,8 @@ describe('instance', function () {
         'isAxiosError',
         'VERSION',
         'default',
-        'toFormData'
+        'toFormData',
+        'formToJSON'
       ].indexOf(prop) > -1) {
         continue;
       }
@@ -34,7 +35,7 @@ describe('instance', function () {
   });
 
   it('should make an http request without verb helper', function (done) {
-    var instance = axios.create();
+    const instance = axios.create();
 
     instance('/foo');
 
@@ -45,7 +46,7 @@ describe('instance', function () {
   });
 
   it('should make an http request with url instead of baseURL', function (done) {
-    var instance = axios.create({
+    const instance = axios.create({
       url: 'https://api.example.com'
     });
 
@@ -58,7 +59,7 @@ describe('instance', function () {
   });
 
   it('should make an http request', function (done) {
-    var instance = axios.create();
+    const instance = axios.create();
 
     instance.get('/foo');
 
@@ -69,7 +70,7 @@ describe('instance', function () {
   });
 
   it('should use instance options', function (done) {
-    var instance = axios.create({ timeout: 1000 });
+    const instance = axios.create({ timeout: 1000 });
 
     instance.get('/foo');
 
@@ -80,7 +81,7 @@ describe('instance', function () {
   });
 
   it('should have defaults.headers', function () {
-    var instance = axios.create({
+    const instance = axios.create({
       baseURL: 'https://api.example.com'
     });
 
@@ -94,13 +95,13 @@ describe('instance', function () {
       return config;
     });
 
-    var instance = axios.create();
+    const instance = axios.create();
     instance.interceptors.request.use(function (config) {
       config.bar = true;
       return config;
     });
 
-    var response;
+    let response;
     instance.get('/foo').then(function (res) {
       response = res;
     });
@@ -119,10 +120,10 @@ describe('instance', function () {
   });
 
   it('should have getUri on the instance', function() {
-    var instance = axios.create({
+    const instance = axios.create({
       baseURL: 'https://api.example.com'
     });
-    var options = {
+    const options = {
       url: 'foo/bar',
       params: {
         name: 'axios'
@@ -132,8 +133,8 @@ describe('instance', function () {
   });
 
   it('should correctly build url without baseURL', function () {
-    var instance = axios.create();
-    var options = {
+    const instance = axios.create();
+    const options = {
       url: 'foo/bar?foo=bar',
       params: {
         name: 'axios'
@@ -143,8 +144,8 @@ describe('instance', function () {
   });
 
   it('should correctly discard url hash mark', function () {
-    var instance = axios.create();
-    var options = {
+    const instance = axios.create();
+    const options = {
       baseURL: 'https://api.example.com',
       url: 'foo/bar?foo=bar#hash',
       params: {
