@@ -1,17 +1,103 @@
-var axios = require('../../../index');
-var http = require('http');
-var https = require('https');
-var net = require('net');
-var url = require('url');
-var zlib = require('zlib');
-var assert = require('assert');
-var fs = require('fs');
-var path = require('path');
-var pkg = require('./../../../package.json');
-var server, proxy;
-var AxiosError = require('../../../lib/core/AxiosError');
-var FormData = require('form-data');
-var formidable = require('formidable');
+import axios from '../../../index.js';
+import http from 'http';
+import https from 'https';
+import net from 'net';
+import url from 'url';
+import zlib from 'zlib';
+import stream from 'stream';
+import util from 'util';
+import assert from 'assert';
+import fs from 'fs';
+import path from 'path';
+let server, proxy;
+import AxiosError from '../../../lib/core/AxiosError.js';
+import FormData from 'form-data';
+import formidable from 'formidable';
+import express from 'express';
+import multer from 'multer';
+import bodyParser from 'body-parser';
+const isBlobSupported = typeof Blob !== 'undefined';
+import {Throttle} from 'stream-throttle';
+import devNull from 'dev-null';
+import {AbortController} from 'abortcontroller-polyfill/dist/cjs-ponyfill.js';
+
+const __filename = url.fileURLToPath(import.meta.url);
+const __dirname = path.dirname(__filename);
+
+import getStream from 'get-stream';
+
+function setTimeoutAsync(ms) {
+  return new Promise(resolve=> setTimeout(resolve, ms));
+}
+
+const pipelineAsync = util.promisify(stream.pipeline);
+const finishedAsync = util.promisify(stream.finished);
+
+function toleranceRange(positive, negative) {
+  const p = (1 + 1 / positive);
+  const n = (1 / negative);
+
+  return (actualValue, value) => {
+    return actualValue - value > 0 ? actualValue < value * p : actualValue > value * n;
+  }
+}
+
+var noop = ()=> {};
+
+const LOCAL_SERVER_URL = 'http://localhost:4444';
+
+function startHTTPServer({useBuffering= false, rate = undefined, port = 4444} = {}) {
+  return new Promise((resolve, reject) => {
+    http.createServer(async function (req, res) {
+      try {
+        req.headers['content-length'] && res.setHeader('content-length', req.headers['content-length']);
+
+        var dataStream = req;
+
+        if (useBuffering) {
+          dataStream = stream.Readable.from(await getStream(req));
+        }
+
+        var streams = [dataStream];
+
+        if (rate) {
+          streams.push(new Throttle({rate}))
+        }
+
+        streams.push(res);
+
+        stream.pipeline(streams, (err) => {
+          err && console.log('Server warning: ' + err.message)
+        });
+      } catch (err){
+        console.warn('HTTP server error:', err);
+      }
+
+    }).listen(port, function (err) {
+      err ? reject(err) : resolve(this);
+    });
+  });
+}
+
+function generateReadableStream(length = 1024 * 1024, chunkSize = 10 * 1024, sleep = 50) {
+  return stream.Readable.from(async function* (){
+    let dataLength = 0;
+
+    while(dataLength < length) {
+      const leftBytes = length - dataLength;
+
+      const chunk = Buffer.alloc(leftBytes > chunkSize? chunkSize : leftBytes);
+
+      dataLength += chunk.length;
+
+      yield chunk;
+
+      if (sleep) {
+        await setTimeoutAsync(sleep);
+      }
+    }
+  }());
+}
 
 describe('supports http with nodejs', function () {
 
@@ -24,12 +110,9 @@ describe('supports http with nodejs', function () {
       proxy.close();
       proxy = null;
     }
-    if (process.env.http_proxy) {
-      delete process.env.http_proxy;
-    }
-    if (process.env.no_proxy) {
-      delete process.env.no_proxy;
-    }
+    delete process.env.http_proxy;
+    delete process.env.https_proxy;
+    delete process.env.no_proxy;
   });
 
   it('should throw an error if the timeout property is not parsable as a number', function (done) {
@@ -143,7 +226,7 @@ describe('supports http with nodejs', function () {
         assert.strictEqual(success, false, 'request should not succeed');
         assert.strictEqual(failure, true, 'request should fail');
         assert.strictEqual(error.code, 'ECONNABORTED');
-        assert.strictEqual(error.message, 'timeout of 250ms exceeded');
+        assert.strictEqual(error.message, 'oops, timeout');
         done();
       }, 300);
     });
@@ -163,7 +246,7 @@ describe('supports http with nodejs', function () {
       axios.get('http://localhost:4444/').then(function (res) {
         assert.deepEqual(res.data, data);
         done();
-      });
+      }).catch(done);
     });
   });
 
@@ -183,7 +266,7 @@ describe('supports http with nodejs', function () {
       axios.get('http://localhost:4444/').then(function (res) {
         assert.deepEqual(res.data, data);
         done();
-      });
+      }).catch(done);;
     });
   });
 
@@ -205,7 +288,7 @@ describe('supports http with nodejs', function () {
         assert.equal(res.data, str);
         assert.equal(res.request.path, '/two');
         done();
-      });
+      }).catch(done);
     });
   });
 
@@ -224,7 +307,7 @@ describe('supports http with nodejs', function () {
         assert.equal(res.status, 302);
         assert.equal(res.headers['location'], '/foo');
         done();
-      });
+      }).catch(done);
     });
   });
 
@@ -242,7 +325,7 @@ describe('supports http with nodejs', function () {
         assert.equal(error.code, AxiosError.ERR_FR_TOO_MANY_REDIRECTS);
         assert.equal(error.message, 'Maximum number of redirects exceeded');
         done();
-      });
+      }).catch(done);
     });
   });
 
@@ -264,6 +347,56 @@ describe('supports http with nodejs', function () {
       }).catch(function (error) {
         assert.equal(error.message, 'Provided path is not allowed');
         done();
+      }).catch(done);
+    });
+  });
+
+  it('should support beforeRedirect and proxy with redirect', function (done) {
+    var requestCount = 0;
+    var totalRedirectCount = 5;
+    server = http.createServer(function (req, res) {
+      requestCount += 1;
+      if (requestCount <= totalRedirectCount) {
+        res.setHeader('Location', 'http://localhost:4444');
+        res.writeHead(302);
+      }
+      res.end();
+    }).listen(4444, function () {
+      var proxyUseCount = 0;
+      proxy = http.createServer(function (request, response) {
+        proxyUseCount += 1;
+        var parsed = url.parse(request.url);
+        var opts = {
+          host: parsed.hostname,
+          port: parsed.port,
+          path: parsed.path
+        };
+
+        http.get(opts, function (res) {
+          response.writeHead(res.statusCode, res.headers);
+          res.on('data', function (data) {
+            response.write(data)
+          });
+          res.on('end', function () {
+            response.end();
+          });
+        });
+      }).listen(4000, function () {
+        var configBeforeRedirectCount = 0;
+        axios.get('http://localhost:4444/', {
+          proxy: {
+            host: 'localhost',
+            port: 4000
+          },
+          maxRedirects: totalRedirectCount,
+          beforeRedirect: function (options) {
+            configBeforeRedirectCount += 1;
+          }
+        }).then(function (res) {
+          assert.equal(totalRedirectCount, configBeforeRedirectCount, 'should invoke config.beforeRedirect option on every redirect');
+          assert.equal(totalRedirectCount + 1, proxyUseCount, 'should go through proxy on every redirect');
+          done();
+        }).catch(done);
       });
     });
   });
@@ -288,9 +421,7 @@ describe('supports http with nodejs', function () {
       axios.head('http://localhost:4444/one').then(function (res) {
         assert.equal(res.status, 200);
         done();
-      }).catch(function (err) {
-        done(err);
-      });
+      }).catch(done);
     });
   });
 
@@ -311,9 +442,8 @@ describe('supports http with nodejs', function () {
         axios.get('http://localhost:4444/').then(function (res) {
           assert.deepEqual(res.data, data);
           done();
-        });
+        }).catch(done);
       });
-
     });
   });
 
@@ -325,7 +455,7 @@ describe('supports http with nodejs', function () {
     }).listen(4444, function () {
       axios.get('http://localhost:4444/').catch(function (error) {
         done();
-      });
+      }).catch(done);
     });
   });
 
@@ -345,7 +475,7 @@ describe('supports http with nodejs', function () {
         }).then(function(res) {
           assert.equal(res.data.toString('base64'), zipped.toString('base64'));
           done();
-        });
+        }).catch(done);
       });
     });
   });
@@ -360,7 +490,7 @@ describe('supports http with nodejs', function () {
       axios.get('http://localhost:4444/').then(function (res) {
         assert.equal(res.data, str);
         done();
-      });
+      }).catch(done);
     });
   });
 
@@ -374,7 +504,7 @@ describe('supports http with nodejs', function () {
         var base64 = Buffer.from(user + ':', 'utf8').toString('base64');
         assert.equal(res.data, 'Basic ' + base64);
         done();
-      });
+      }).catch(done);
     });
   });
 
@@ -388,7 +518,7 @@ describe('supports http with nodejs', function () {
         var base64 = Buffer.from('foo:bar', 'utf8').toString('base64');
         assert.equal(res.data, 'Basic ' + base64);
         done();
-      });
+      }).catch(done);
     });
   });
 
@@ -397,9 +527,9 @@ describe('supports http with nodejs', function () {
       res.end(req.headers['user-agent']);
     }).listen(4444, function () {
       axios.get('http://localhost:4444/').then(function (res) {
-        assert.ok(/^axios\/[\d.]+$/.test(res.data), `User-Agent header does not match: ${res.data}`);
+        assert.ok(/^axios\/[\d.]+[-]?[a-z]*[.]?[\d]+$/.test(res.data), `User-Agent header does not match: ${res.data}`);
         done();
-      });
+      }).catch(done);
     });
   });
 
@@ -411,7 +541,7 @@ describe('supports http with nodejs', function () {
       axios.get('http://localhost:4444/', { headers }).then(function (res) {
         assert.equal(res.data, 'foo bar');
         done();
-      });
+      }).catch(done);
     });
   });
 
@@ -423,7 +553,7 @@ describe('supports http with nodejs', function () {
       var headers = { 'CoNtEnT-lEnGtH': '42' }; // wonky casing to ensure caseless comparison
       axios.post('http://localhost:4444/', 'foo', { headers }).then(function () {
         done();
-      });
+      }).catch(done);
     });
   });
 
@@ -519,6 +649,32 @@ describe('supports http with nodejs', function () {
     });
   });
 
+  it('should properly support default max body length (follow-redirects as well)', function (done) {
+    // taken from https://github.com/follow-redirects/follow-redirects/blob/22e81fc37132941fb83939d1dc4c2282b5c69521/index.js#L461
+    var followRedirectsMaxBodyDefaults = 10 * 1024 *1024;
+    var data = Array(2 * followRedirectsMaxBodyDefaults).join('ж');
+
+    server = http.createServer(function (req, res) {
+      // consume the req stream
+      req.on('data', noop);
+      // and wait for the end before responding, otherwise an ECONNRESET error will be thrown
+      req.on('end', ()=> {
+        res.end('OK');
+      });
+    }).listen(4444, function (err) {
+      if (err) {
+        return done(err);
+      }
+      // send using the default -1 (unlimited axios maxBodyLength)
+      axios.post('http://localhost:4444/', {
+        data: data
+      }).then(function (res) {
+        assert.equal(res.data, 'OK', 'should handle response');
+        done();
+      }).catch(done);
+    });
+  });
+
   it('should display error while parsing params', function (done) {
     server = http.createServer(function () {
 
@@ -530,7 +686,7 @@ describe('supports http with nodejs', function () {
       }).catch(function (err) {
         assert.deepEqual(err.exists, true)
         done();
-      });
+      }).catch(done);
     });
   });
 
@@ -549,28 +705,25 @@ describe('supports http with nodejs', function () {
     }).listen(socketName, function () {
       axios({
         socketPath: socketName,
-        url: '/'
+        url: 'http://localhost:4444/socket'
       })
         .then(function (resp) {
           assert.equal(resp.status, 200);
           assert.equal(resp.statusText, 'OK');
           done();
-        })
-        .catch(function (error) {
-          assert.ifError(error);
-          done();
-        });
+        }).catch(done);
     });
   });
 
-  it('should support streams', function (done) {
-    server = http.createServer(function (req, res) {
-      req.pipe(res);
-    }).listen(4444, function () {
-      axios.post('http://localhost:4444/',
-        fs.createReadStream(__filename), {
-          responseType: 'stream'
-        }).then(function (res) {
+  describe('streams', function(){
+    it('should support streams', function (done) {
+      server = http.createServer(function (req, res) {
+        req.pipe(res);
+      }).listen(4444, function () {
+        axios.post('http://localhost:4444/',
+          fs.createReadStream(__filename), {
+            responseType: 'stream'
+          }).then(function (res) {
           var stream = res.data;
           var string = '';
           stream.on('data', function (chunk) {
@@ -580,25 +733,51 @@ describe('supports http with nodejs', function () {
             assert.equal(string, fs.readFileSync(__filename, 'utf8'));
             done();
           });
-        });
-    });
-  });
-
-  it('should pass errors for a failed stream', function (done) {
-    var notExitPath = path.join(__dirname, 'does_not_exist');
-
-    server = http.createServer(function (req, res) {
-      req.pipe(res);
-    }).listen(4444, function () {
-      axios.post('http://localhost:4444/',
-        fs.createReadStream(notExitPath)
-      ).then(function (res) {
-        assert.fail();
-      }).catch(function (err) {
-        assert.equal(err.message, `ENOENT: no such file or directory, open \'${notExitPath}\'`);
-        done();
+        }).catch(done);
       });
     });
+
+    it('should pass errors for a failed stream', async function () {
+      server = await startHTTPServer();
+
+      var notExitPath = path.join(__dirname, 'does_not_exist');
+
+      try {
+        await axios.post(LOCAL_SERVER_URL,
+          fs.createReadStream(notExitPath)
+        );
+        assert.fail('expected ENOENT error');
+      } catch (err) {
+        assert.equal(err.message, `ENOENT: no such file or directory, open \'${notExitPath}\'`);
+      }
+    });
+
+    it('should destroy the response stream with an error on request stream destroying', async function () {
+      server = await startHTTPServer();
+
+      let stream = generateReadableStream();
+
+      setTimeout(function () {
+        stream.destroy();
+      }, 1000);
+
+      const {data} = await axios.post(LOCAL_SERVER_URL, stream, {responseType: 'stream'});
+
+      let streamError;
+
+      data.on('error', function(err) {
+        streamError = err;
+      });
+
+      try {
+        await pipelineAsync(data, devNull());
+        assert.fail('stream was not aborted');
+      } catch(e) {
+        console.log(`pipeline error: ${e}`);
+      } finally {
+        assert.strictEqual(streamError && streamError.code, 'ERR_CANCELED');
+      }
+    });
   });
 
   it('should support buffers', function (done) {
@@ -620,7 +799,7 @@ describe('supports http with nodejs', function () {
             assert.equal(string, buf.toString());
             done();
           });
-        });
+        }).catch(done);
     });
   });
 
@@ -657,7 +836,7 @@ describe('supports http with nodejs', function () {
         }).then(function (res) {
           assert.equal(res.data, '123456789', 'should pass through proxy');
           done();
-        });
+        }).catch(done);
       });
     });
   });
@@ -697,7 +876,7 @@ describe('supports http with nodejs', function () {
           proxy: {
             host: 'localhost',
             port: 4000,
-            protocol: 'https'
+            protocol: 'https:'
           },
           httpsAgent: new https.Agent({
             rejectUnauthorized: false
@@ -705,10 +884,7 @@ describe('supports http with nodejs', function () {
         }).then(function (res) {
           assert.equal(res.data, '123456789', 'should pass through proxy');
           done();
-        }).catch(function (err) {
-          assert.fail(err);
-          done()
-        });
+        }).catch(done);
       });
     });
   });
@@ -726,7 +902,7 @@ describe('supports http with nodejs', function () {
       }).then(function (res) {
         assert.equal(res.data, '123456789', 'should not pass through proxy');
         done();
-      });
+      }).catch(done);
     });
   });
 
@@ -761,7 +937,7 @@ describe('supports http with nodejs', function () {
         axios.get('http://localhost:4444/').then(function (res) {
           assert.equal(res.data, '45671234', 'should use proxy set by process.env.http_proxy');
           done();
-        });
+        }).catch(done);
       });
     });
   });
@@ -806,12 +982,57 @@ describe('supports http with nodejs', function () {
         }).then(function (res) {
           assert.equal(res.data, '123456789', 'should pass through proxy');
           done();
-        }).catch(function (err) {
-          assert.fail(err);
-          done()
-        }).finally(function () {
-          process.env.https_proxy = ''
+        }).catch(done);
+      });
+    });
+  });
+
+  it('should re-evaluate proxy on redirect when proxy set via env var', function (done) {
+    process.env.http_proxy = 'http://localhost:4000'
+    process.env.no_proxy = 'localhost:4000'
+
+    var proxyUseCount = 0;
+
+    server = http.createServer(function (req, res) {
+      res.setHeader('Location', 'http://localhost:4000/redirected');
+      res.statusCode = 302;
+      res.end();
+    }).listen(4444, function () {
+      proxy = http.createServer(function (request, response) {
+        var parsed = url.parse(request.url);
+        if (parsed.pathname === '/redirected') {
+          response.statusCode = 200;
+          response.end();
+          return;
+        }
+
+        proxyUseCount += 1;
+
+        var opts = {
+          host: parsed.hostname,
+          port: parsed.port,
+          path: parsed.path,
+          protocol: parsed.protocol,
+          rejectUnauthorized: false
+        };
+
+        http.get(opts, function (res) {
+          var body = '';
+          res.on('data', function (data) {
+            body += data;
+          });
+          res.on('end', function () {
+            response.setHeader('Content-Type', 'text/html; charset=UTF-8');
+            response.setHeader('Location', res.headers.location);
+            response.end(body);
+          });
         });
+      }).listen(4000, function () {
+        axios.get('http://localhost:4444/').then(function(res) {
+          assert.equal(res.status, 200);
+          assert.equal(proxyUseCount, 1);
+          done();
+        }).catch(done);
       });
     });
   });
@@ -848,7 +1069,7 @@ describe('supports http with nodejs', function () {
         axios.get('http://localhost:4444/').then(function (res) {
           assert.equal(res.data, '4567', 'should not use proxy for domains in no_proxy');
           done();
-        });
+        }).catch(done);
       });
     });
   });
@@ -885,7 +1106,7 @@ describe('supports http with nodejs', function () {
         axios.get('http://localhost:4444/').then(function (res) {
           assert.equal(res.data, '45671234', 'should use proxy for domains not in no_proxy');
           done();
-        });
+        }).catch(done);
       });
     });
   });
@@ -928,7 +1149,7 @@ describe('supports http with nodejs', function () {
           var base64 = Buffer.from('user:pass', 'utf8').toString('base64');
           assert.equal(res.data, 'Basic ' + base64, 'should authenticate to the proxy');
           done();
-        });
+        }).catch(done);
       });
     });
   });
@@ -964,7 +1185,7 @@ describe('supports http with nodejs', function () {
           var base64 = Buffer.from('user:pass', 'utf8').toString('base64');
           assert.equal(res.data, 'Basic ' + base64, 'should authenticate to the proxy set by process.env.http_proxy');
           done();
-        });
+        }).catch(done);
       });
     });
   });
@@ -1010,7 +1231,7 @@ describe('supports http with nodejs', function () {
           var base64 = Buffer.from('user:pass', 'utf8').toString('base64');
           assert.equal(res.data, 'Basic ' + base64, 'should authenticate to the proxy');
           done();
-        });
+        }).catch(done);
       });
     });
   });
@@ -1041,7 +1262,7 @@ describe('supports http with nodejs', function () {
         assert.equal(res.config.baseURL, 'http://localhost:4444/');
         assert.equal(res.config.url, '/foo');
         done();
-      });
+      }).catch(done);
     });
   });
 
@@ -1127,13 +1348,13 @@ describe('supports http with nodejs', function () {
 
   it('should supply a user-agent if one is not specified', function (done) {
     server = http.createServer(function (req, res) {
-      assert.equal(req.headers["user-agent"], 'axios/' + pkg.version);
+      assert.equal(req.headers["user-agent"], 'axios/' + axios.VERSION);
       res.end();
     }).listen(4444, function () {
       axios.get('http://localhost:4444/'
       ).then(function (res) {
         done();
-      });
+      }).catch(done);
     });
   });
 
@@ -1147,10 +1368,9 @@ describe('supports http with nodejs', function () {
         headers: {
           "User-Agent": null
         }
-      }
-      ).then(function (res) {
+      }).then(function (res) {
         done();
-      });
+      }).catch(done);
     });
   });
 
@@ -1175,56 +1395,500 @@ describe('supports http with nodejs', function () {
       }).catch(function (err) {
         error = err;
         failure = true;
-      }).finally(function () {
+      }).then(function () {
         assert.strictEqual(success, false, 'request should not succeed');
         assert.strictEqual(failure, true, 'request should fail');
         assert.strictEqual(error.code, 'ERR_BAD_RESPONSE');
         assert.strictEqual(error.message, 'maxContentLength size of -1 exceeded');
-        done();
-      });
-    });
-  });
-
-  it('should allow passing FormData', function (done) {
-    var form = new FormData();
-    var file1= Buffer.from('foo', 'utf8');
-
-    form.append('foo', "bar");
-    form.append('file1', file1, {
-      filename: 'bar.jpg',
-      filepath: 'temp/bar.jpg',
-      contentType: 'image/jpeg'
-    });
-
-    server = http.createServer(function (req, res) {
-      var receivedForm = new formidable.IncomingForm();
-
-      receivedForm.parse(req, function (err, fields, files) {
-        if (err) {
-          return done(err);
-        }
-
-        res.end(JSON.stringify({
-          fields: fields,
-          files: files
-        }));
-      });
-    }).listen(4444, function () {
-      axios.post('http://localhost:4444/', form, {
-        headers: {
-          'Content-Type': 'multipart/form-data'
-        }
-      }).then(function (res) {
-        assert.deepStrictEqual(res.data.fields,{foo: 'bar'});
-
-        assert.strictEqual(res.data.files.file1.mimetype,'image/jpeg');
-        assert.strictEqual(res.data.files.file1.originalFilename,'temp/bar.jpg');
-        assert.strictEqual(res.data.files.file1.size,3);
-
         done();
       }).catch(done);
     });
   });
 
-});
+  it('should able to cancel multiple requests with CancelToken', function(done){
+    server = http.createServer(function (req, res) {
+      res.end('ok');
+    }).listen(4444, function () {
+      var CancelToken = axios.CancelToken;
+      var source = CancelToken.source();
+      var canceledStack = [];
 
+      var requests = [1, 2, 3, 4, 5].map(function(id){
+        return axios
+          .get('/foo/bar', { cancelToken: source.token })
+          .catch(function (e) {
+            if (!axios.isCancel(e)) {
+              throw e;
+            }
+
+            canceledStack.push(id);
+          });
+      });
+
+      source.cancel("Aborted by user");
+
+      Promise.all(requests).then(function () {
+        assert.deepStrictEqual(canceledStack.sort(), [1, 2, 3, 4, 5])
+      }).then(done, done);
+    });
+  });
+
+  describe('FormData', function () {
+    it('should allow passing FormData', function (done) {
+      var form = new FormData();
+      var file1 = Buffer.from('foo', 'utf8');
+
+      form.append('foo', "bar");
+      form.append('file1', file1, {
+        filename: 'bar.jpg',
+        filepath: 'temp/bar.jpg',
+        contentType: 'image/jpeg'
+      });
+
+      server = http.createServer(function (req, res) {
+        var receivedForm = new formidable.IncomingForm();
+
+        receivedForm.parse(req, function (err, fields, files) {
+          if (err) {
+            return done(err);
+          }
+
+          res.end(JSON.stringify({
+            fields: fields,
+            files: files
+          }));
+        });
+      }).listen(4444, function () {
+        axios.post('http://localhost:4444/', form, {
+          headers: {
+            'Content-Type': 'multipart/form-data'
+          }
+        }).then(function (res) {
+          assert.deepStrictEqual(res.data.fields, {foo: 'bar'});
+
+          assert.strictEqual(res.data.files.file1.mimetype, 'image/jpeg');
+          assert.strictEqual(res.data.files.file1.originalFilename, 'temp/bar.jpg');
+          assert.strictEqual(res.data.files.file1.size, 3);
+
+          done();
+        }).catch(done);
+      });
+    });
+
+    describe('toFormData helper', function () {
+      it('should properly serialize nested objects for parsing with multer.js (express.js)', function (done) {
+        var app = express();
+
+        var obj = {
+          arr1: ['1', '2', '3'],
+          arr2: ['1', ['2'], '3'],
+          obj: {x: '1', y: {z: '1'}},
+          users: [{name: 'Peter', surname: 'griffin'}, {name: 'Thomas', surname: 'Anderson'}]
+        };
+
+        app.post('/', multer().none(), function (req, res, next) {
+          res.send(JSON.stringify(req.body));
+        });
+
+        server = app.listen(3001, function () {
+          // multer can parse the following key/value pairs to an array (indexes: null, false, true):
+          // arr: '1'
+          // arr: '2'
+          // -------------
+          // arr[]: '1'
+          // arr[]: '2'
+          // -------------
+          // arr[0]: '1'
+          // arr[1]: '2'
+          // -------------
+          Promise.all([null, false, true].map(function (mode) {
+            return axios.postForm('http://localhost:3001/', obj, {formSerializer: {indexes: mode}})
+              .then(function (res) {
+                assert.deepStrictEqual(res.data, obj, 'Index mode ' + mode);
+              });
+          })).then(function (){
+            done();
+          }, done)
+        });
+      });
+    });
+  });
+
+  describe('URLEncoded Form', function () {
+    it('should post object data as url-encoded form if content-type is application/x-www-form-urlencoded', function (done) {
+      var app = express();
+
+      var obj = {
+        arr1: ['1', '2', '3'],
+        arr2: ['1', ['2'], '3'],
+        obj: {x: '1', y: {z: '1'}},
+        users: [{name: 'Peter', surname: 'griffin'}, {name: 'Thomas', surname: 'Anderson'}]
+      };
+
+      app.use(bodyParser.urlencoded({ extended: true }));
+
+      app.post('/', function (req, res, next) {
+        res.send(JSON.stringify(req.body));
+      });
+
+      server = app.listen(3001, function () {
+        return axios.post('http://localhost:3001/', obj, {
+          headers: {
+            'content-type': 'application/x-www-form-urlencoded'
+          }
+        })
+          .then(function (res) {
+            assert.deepStrictEqual(res.data, obj);
+            done();
+          }).catch(done);
+      });
+    });
+  });
+
+  it('should respect formSerializer config', function (done) {
+    const obj = {
+      arr1: ['1', '2', '3'],
+      arr2: ['1', ['2'], '3'],
+    };
+
+    const form = new URLSearchParams();
+
+    form.append('arr1[0]', '1');
+    form.append('arr1[1]', '2');
+    form.append('arr1[2]', '3');
+
+    form.append('arr2[0]', '1');
+    form.append('arr2[1][0]', '2');
+    form.append('arr2[2]', '3');
+
+    server = http.createServer(function (req, res) {
+      req.pipe(res);
+    }).listen(3001, () => {
+      return axios.post('http://localhost:3001/', obj, {
+        headers: {
+          'content-type': 'application/x-www-form-urlencoded'
+        },
+        formSerializer: {
+          indexes: true
+        }
+      })
+        .then(function (res) {
+          assert.strictEqual(res.data, form.toString());
+          done();
+        }).catch(done);
+    });
+  });
+
+  describe('Data URL', function () {
+    it('should support requesting data URL as a Buffer', function (done) {
+      const buffer = Buffer.from('123');
+
+      const dataURI = 'data:application/octet-stream;base64,' + buffer.toString('base64');
+
+      axios.get(dataURI).then(({data})=> {
+        assert.deepStrictEqual(data, buffer);
+        done();
+      }).catch(done);
+    });
+
+    it('should support requesting data URL as a Blob (if supported by the environment)', function (done) {
+
+      if (!isBlobSupported) {
+        this.skip();
+        return;
+      }
+
+      const buffer = Buffer.from('123');
+
+      const dataURI = 'data:application/octet-stream;base64,' + buffer.toString('base64');
+
+      axios.get(dataURI, {responseType: 'blob'}).then(async ({data})=> {
+        assert.strictEqual(data.type, 'application/octet-stream');
+        assert.deepStrictEqual(await data.text(), '123');
+        done();
+      }).catch(done);
+    });
+
+    it('should support requesting data URL as a String (text)', function (done) {
+      const buffer = Buffer.from('123', 'utf-8');
+
+      const dataURI = 'data:application/octet-stream;base64,' + buffer.toString('base64');
+
+      axios.get(dataURI, {responseType: "text"}).then(({data})=> {
+        assert.deepStrictEqual(data, '123');
+        done();
+      }).catch(done);
+    });
+
+    it('should support requesting data URL as a Stream', function (done) {
+      const buffer = Buffer.from('123', 'utf-8');
+
+      const dataURI = 'data:application/octet-stream;base64,' + buffer.toString('base64');
+
+      axios.get(dataURI, {responseType: "stream"}).then(({data})=> {
+        var str = '';
+
+        data.on('data', function(response){
+          str += response.toString();
+        });
+
+        data.on('end', function(){
+          assert.strictEqual(str, '123');
+          done();
+        });
+      }).catch(done);
+    });
+  });
+
+  describe('progress', function () {
+    describe('upload', function () {
+      it('should support upload progress capturing', async function () {
+        server = await startHTTPServer({
+          rate: 100 * 1024
+        });
+
+        let content = '';
+        const count = 10;
+        const chunk = "test";
+        const chunkLength = Buffer.byteLength(chunk);
+        const contentLength = count * chunkLength;
+
+        var readable = stream.Readable.from(async function* () {
+          let i = count;
+
+          while (i-- > 0) {
+            await setTimeoutAsync(1100);
+            content += chunk;
+            yield chunk;
+          }
+        }());
+
+        const samples = [];
+
+        const {data} = await axios.post(LOCAL_SERVER_URL, readable, {
+          onUploadProgress: ({loaded, total, progress, bytes, upload}) => {
+            samples.push({
+              loaded,
+              total,
+              progress,
+              bytes,
+              upload
+            });
+          },
+          headers: {
+            'Content-Length': contentLength
+          },
+          responseType: 'text'
+        });
+
+        assert.strictEqual(data, content);
+
+        assert.deepStrictEqual(samples, Array.from(function* () {
+          for (let i = 1; i <= 10; i++) {
+            yield ({
+              loaded: chunkLength * i,
+              total: contentLength,
+              progress: (chunkLength * i) / contentLength,
+              bytes: 4,
+              upload: true
+            });
+          }
+        }()));
+      });
+    });
+
+    describe('download', function () {
+      it('should support download progress capturing', async function () {
+        server = await startHTTPServer({
+          rate: 100 * 1024
+        });
+
+        let content = '';
+        const count = 10;
+        const chunk = "test";
+        const chunkLength = Buffer.byteLength(chunk);
+        const contentLength = count * chunkLength;
+
+        var readable = stream.Readable.from(async function* () {
+          let i = count;
+
+          while (i-- > 0) {
+            await setTimeoutAsync(1100);
+            content += chunk;
+            yield chunk;
+          }
+        }());
+
+        const samples = [];
+
+        const {data} = await axios.post(LOCAL_SERVER_URL, readable, {
+          onDownloadProgress: ({loaded, total, progress, bytes, download}) => {
+            samples.push({
+              loaded,
+              total,
+              progress,
+              bytes,
+              download
+            });
+          },
+          headers: {
+            'Content-Length': contentLength
+          },
+          responseType: 'text',
+          maxRedirects: 0
+        });
+
+        assert.strictEqual(data, content);
+
+        assert.deepStrictEqual(samples, Array.from(function* () {
+          for (let i = 1; i <= 10; i++) {
+            yield ({
+              loaded: chunkLength * i,
+              total: contentLength,
+              progress: (chunkLength * i) / contentLength,
+              bytes: 4,
+              download: true
+            });
+          }
+        }()));
+      });
+    });
+  });
+
+  describe('Rate limit', function () {
+    it('should support upload rate limit', async function () {
+      const secs = 10;
+      const configRate = 100_000;
+      const chunkLength = configRate * secs;
+
+      server = await startHTTPServer();
+
+      const buf = Buffer.alloc(chunkLength).fill('s');
+      const samples = [];
+      const skip = 2;
+      const compareValues = toleranceRange(10, 50);
+
+      const {data} = await axios.post(LOCAL_SERVER_URL, buf, {
+        onUploadProgress: ({loaded, total, progress, bytes, rate}) => {
+          samples.push({
+            loaded,
+            total,
+            progress,
+            bytes,
+            rate
+          });
+        },
+        maxRate: [configRate],
+        responseType: 'text',
+        maxRedirects: 0
+      });
+
+      samples.slice(skip).forEach(({rate, progress}, i, _samples)=> {
+        assert.ok(compareValues(rate, configRate),
+          `Rate sample at index ${i} is out of the expected range (${rate} / ${configRate}) [${
+            _samples.map(({rate}) => rate).join(', ')
+          }]`
+        );
+
+        const progressTicksRate = 2;
+        const expectedProgress = ((i + skip) / secs) / progressTicksRate;
+
+        assert.ok(
+          Math.abs(expectedProgress - progress) < 0.25,
+          `Progress sample at index ${i} is out of the expected range (${progress} / ${expectedProgress}) [${
+            _samples.map(({progress}) => progress).join(', ')
+          }]`
+        );
+      });
+
+      assert.strictEqual(data, buf.toString(), 'content corrupted');
+    });
+
+    it('should support download rate limit', async function () {
+      const secs = 10;
+      const configRate = 100_000;
+      const chunkLength = configRate * secs;
+
+      server = await startHTTPServer();
+
+      const buf = Buffer.alloc(chunkLength).fill('s');
+      const samples = [];
+      const skip = 2;
+      const compareValues = toleranceRange(10, 50);
+
+      const {data} = await axios.post(LOCAL_SERVER_URL, buf, {
+        onDownloadProgress: ({loaded, total, progress, bytes, rate}) => {
+          samples.push({
+            loaded,
+            total,
+            progress,
+            bytes,
+            rate
+          });
+        },
+        maxRate: [0, configRate],
+        responseType: 'text',
+        maxRedirects: 0
+      });
+
+      samples.slice(skip).forEach(({rate, progress}, i, _samples)=> {
+        assert.ok(compareValues(rate, configRate),
+          `Rate sample at index ${i} is out of the expected range (${rate} / ${configRate}) [${
+            _samples.map(({rate}) => rate).join(', ')
+          }]`
+        );
+
+        const progressTicksRate = 2;
+        const expectedProgress = ((i + skip) / secs) / progressTicksRate;
+
+        assert.ok(
+          Math.abs(expectedProgress - progress) < 0.25,
+          `Progress sample at index ${i} is out of the expected range (${progress} / ${expectedProgress}) [${
+            _samples.map(({progress}) => progress).join(', ')
+          }]`
+        );
+      });
+
+      assert.strictEqual(data, buf.toString(), 'content corrupted');
+    });
+  });
+
+
+  describe('request aborting', function() {
+    it('should be able to abort the response stream', async function () {
+      server = await startHTTPServer({
+        rate: 100_000,
+        useBuffering: true
+      });
+
+      const buf = Buffer.alloc(1024 * 1024);
+
+      const controller = new AbortController();
+
+      var {data} = await axios.post(LOCAL_SERVER_URL, buf, {
+        responseType: 'stream',
+        signal: controller.signal,
+        maxRedirects: 0
+      });
+
+      setTimeout(() => {
+        controller.abort();
+      }, 500);
+
+      let streamError;
+
+      data.on('error', function(err) {
+        streamError = err;
+      });
+
+      try {
+        await pipelineAsync(data, devNull());
+        assert.fail('stream was not aborted');
+      } catch(e) {
+        console.log(`pipeline error: ${e}`);
+      } finally {
+        assert.strictEqual(streamError && streamError.code, 'ERR_CANCELED');
+      }
+    });
+  })
+});
